# Things That Need To Be Done

## Actix Web 4.0
It's not out yet, and not everything has a beta release yet, but we should continue tracking the
status of 4.0 and update our libraries as we can.

## Async
A lot of logic in the Profiles and Submissions modules synchronously accesses Sled, which is an
issue because Sled is potentially blocking. All of the Big Dangers (transactions) are safely on a
threadpool already, but a couple iterations are currently occuring directly on the Arbiters.

## Organization
A lot of modules in hyaenidae-server have circular dependencies, and a few are more than a thousand
lines long, each. These should be separated where possible into smaller modules which create a
dependency DAG

## Record Caching
Although Sled itself has in-memory caching for data, anything that requires
serialization/deserialization should be read at most once when in a route. This can be done in a
manner similar to the FederationView in the admin module. We build HashMaps mapping object IDs to
the Objects, and construct Vecs of Object IDs to denote ordering of objects. The view then produces
iterators over references to Objects by iterating over the IDs and fetching objects from the
HashMap.

#### Example
```rust
struct SubmissionView<'a> {
    author: &'a Profile,
    submission: &'a Submission,
}

impl<'a> SubmissionView<'a> {
    fn whatever_method(&self) -> String {
        ...
    }
}

struct ViewState {
    profiles: HashMap<Uuid, Profile>,
    submissions: HashMap<Uuid, Submission>,
    ordered: Vec<Uuid>,
}

impl ViewState {
    fn submissions<'a>(&'a self) -> impl Iterator<Item = SubmissionView<'a>> + 'a {
        self.ordered.iter().filter_map(move |id| {
            let submission = self.submissions.get(id)?;
            let author = self.profiles.get(&submission.profile_id())?;

            Some(SubmissionView { author, submission })
        })
    }
}
```

This pattern allows for each Object to be stored at most once during a route, and it also avoids
cloning when multiple submissions were posted by the same profile, for example.

## Toolkit
Currently, the Submission Boxes shown on the home page and profile page are defined by
hyaenidae-server. This is potentially a future issue for Themeing, because the colors are not set by
the toolkit.

Tasks:
- Create a grid/hover-overlay component in the toolkit and use it to implement the submission boxes

## Pagination
Right now, the only things that paginate are Submissions. This could potentially cause issues on the
Notifications page, or the Admin page.

Tasks:
- Enable pagination for Known Servers, Federated Servers, Federation Requests,
- Enable pagination for Reports
- (potentially) Enable pagination for comments on submissions
    - this one is less clear because comments are tree-structures

## Submissions
Right now, following doesn't do much of anything from a user perspective. A follower can view
followers-only posts on the author's Profile, or on the Home Page, and following a remote profile
enables that profile to federate new submissions to the local server.

Followers-Only submissions also exist on the back-end, but it is not yet exposed in the UI.
Originally, this was blocked on adding a select component to the toolkit, but that blocker has gone
away.

There's currently no mechanism for marking submissions as NSFW, or for choosing to show or hide NSFW
conent on Hyaenidae. This is a problem for scenarios in which some users wish to post NSFW content
while others wish not to see it.

Tasks:
- Create a Submission View in which submission from followed profiles can be viewed in publish
    order
- Expose Submission Privacy Settings on the Submission Update page for unpublished submissions. It
    does not make sense to allow changing submission privacy after a submission has been published.
- Create a flag for submissions to indicate the presence of NSFW images
- Expose NSFW flag on the Submission Update page
- Expose a setting to show/hide NSFW content to the profile page
- Add "Mark as Sensitive" to report actions
- Ensure submission updates cannot override an administrator's "mark as sensitive" ruling
- Expose NSFW flag to ActivityPub via the 'sensitive' extension used by Mastodon
- (potentially) expose an NSFW flag on profiles to both hide profile banner image, and to default
    submissions to NSFW

## Registration
There is currently no way to disable or limit registration to Hyaenidae

Tasks:
- Add an option in the Admin settings to disable registration
- Create a concept of Invite Links which can be used N (user specified) times to register accounts
    when registration is disabled
- Add an option in the Admin settings to generate invite links

## Tags
Tags are useful for searching Hyaenidae for content. There should be a tag system that makes it
simple to search for submissions.

Tasks:
- Design a Tag database optimized for multi-tag lookups (within reason)
- Design a UI for authors to add tags to a submission
- Design ActivityPub interop for tag federation

## Reports
Reports are implemented for on-server moderation, but cross-server moderation has not been tested.

Tasks:
- Add a "delete" mechanism for remote objects. This should not induce any federation, but should
    ensure the object is not accessible locally
- Enable forwarding reports to origin server. There is already code written to perform this
    forwarding, but it is not exposed in the UI.
- Enable viewing of closed reports

## Reacts
Reacts are similar to "likes" on other platforms, but do not get added to a collection for a given
profile. Reacts only notify the author of the item being reacted to.

Tasks:
- Design a UI for reacting to submissions
- Design a UI for reacting to comments
- Design a UI for reacting to posts (see Posts)
- Decide which kinds of reacts are valid

## Posts
Posts are top-level content created by profiles that do not have associated images, and so cannot be
considered Submissions. Posts are stored and viewed separately from submissions, and are more
"ephemeral" in that they will not backfill (see Backfills).

Tasks:
- Design a Post database
- Design a UI for creating new posts
- Design a UI for viewing posts from followed accounts
- Hook Posts into Note federation code

## Direct Messages
Direct Messages are Posts that are shared directly between two profiles, and are not publically
visible to other profiles.

## Shares
Shares are a collection of submissions a profile can display that were submitted by other profiles.
They will not appear in followers' feeds, but will be visible on a given profile's page. This
feature is analogous to a public favorites gallery on other platforms, although it is separate from
"reacts", which can act as favoriting a post. Whether a profile is notified of a share is undecided.

Tasks:
- Add shares tree(s) in the data store. This is likely not much more than a single tree with a key
    consisting of the Profile ID, a timestamp, and the Submission ID, where the value of is the
    Submission ID as well. A second tree could contain Profile ID + Submission ID keys for quick
    lookups for whether a given submission has been shared by a given profile to avoid duplicates
- Add a shares page or section to the profile section of Hyaenidae.
- Implement Shares in ActivityPub as a variant of Announce Note where the Note represents a
    Submission

## Backfills
Bakfills for a given item should not be permitted to run concurrently. A single backfill task may be
permitted to have multiple requests in-flight, but no two tasks may backfill the same item. There
should also be a limit on concurrency for backfills from a given server

Tasks:
- Add Profile Backfill on federation
- Add Submission Backfill on follow

## Object Cleanup
When things get deleted or suspended, we need to ensure that the database does not hold onto any
lingering data. This includes cleaning text fields for deleted items, cleaning up dependent items on
deletion.

Tasks:
- Differentiate between Profile Deletion and Profile Suspsension
- Federate profile suspensions for local suspended profiles
- Delete profiles (and submissions and comments and reacts) for defederated servers

## Federation Fixes
Not all of federation has been tested yet

Things that have been tested:
- Federation:
    - Request
    - Accept
    - Reject
    - Defederate
- Profiles:
    - Create
    - Update
- Submissions:
    - Create
    - Update

Anything not in this list has not been tested and needs to be verified and possibly fixed.

## Blind Key Rotation
If an incoming post has an invalid key, we should try fetching the key/profile & then verifying the
signature again. This is because other softwares, like mastodon, work on the assumption that you
might not want deleted posts to be associated with your account. It's not a Big Deal IMO when only
HTTP Signatures are in use, such as in Hyaenidae, but when JSON-LD Signatures are in use by other
software, it becomes trivial to trace authorship of any object.
